"""Ceval-Valid group task manifest."""

from __future__ import annotations

BASE_IMPORT = "wisent.extractors.lm_eval.lm_task_extractors."

CEVAL_VALID_TASKS = {
    "ceval-valid": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_accountant": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_advanced_mathematics": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_art_studies": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_basic_medicine": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_business_administration": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_chinese_language_and_literature": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_civil_servant": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_clinical_medicine": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_college_chemistry": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_college_economics": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_college_physics": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_college_programming": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_computer_architecture": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_computer_network": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_discrete_mathematics": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_education_science": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_electrical_engineer": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_environmental_impact_assessment_engineer": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_fire_engineer": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_high_school_biology": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_high_school_chemistry": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_high_school_chinese": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_high_school_geography": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_high_school_history": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_high_school_mathematics": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_high_school_physics": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_high_school_politics": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_ideological_and_moral_cultivation": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_law": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_legal_professional": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_logic": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_mao_zedong_thought": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_marxism": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_metrology_engineer": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_middle_school_biology": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_middle_school_chemistry": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_middle_school_geography": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_middle_school_history": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_middle_school_mathematics": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_middle_school_physics": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_middle_school_politics": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_modern_chinese_history": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_operating_system": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_physician": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_plant_protection": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_probability_and_statistics": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_professional_tour_guide": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_sports_science": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_tax_accountant": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_teacher_qualification": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_urban_and_rural_planner": f"{BASE_IMPORT}ceval:CevalExtractor",
    "ceval-valid_veterinary_medicine": f"{BASE_IMPORT}ceval:CevalExtractor",
}
