import os as _os
_base = _os.path.dirname(__file__)
for _entry in sorted(_os.listdir(_base)):
    _path = _os.path.join(_base, _entry)
    if _os.path.isdir(_path) and not _entry.startswith(('.', '_')):
        __path__.append(_path)

"""Agent implementation modules."""

from .agent_helpers.generate_synthetic_pairs import generate_synthetic_pairs
from .agent_helpers.train_classifier import train_classifier_on_pairs
from .evaluate_response import evaluate_response_with_classifier
from .apply_steering import apply_steering_and_evaluate
from .main import execute_agent

__all__ = [
    "generate_synthetic_pairs",
    "train_classifier_on_pairs",
    "evaluate_response_with_classifier",
    "apply_steering_and_evaluate",
    "execute_agent",
]
